;*************************************************************************
;*        .
;* FUSES: CKSEL=1101, SUT=11
;*************************************************************************

.nolist
.include "tn2313def.inc"
.list

;*****  *****


.def	Save = r12         ;   SREG  

.def	tmp1	 = r16     ;   
.def	tmp2	 = r17     ;   
.def	spi_byte = r18     ;       I2C

.def    CntSel = r23   ;   
.def	time = r24     ;   
.def    Cnt = r25      ;         

;***** *****

.equ    sck	= PD2       ; Shift-Clock =  D /  2
.equ    mosi= PD1   	; MOSI =  D /  1

.cseg

.org 0
	rjmp Initial

.org 1
	rjmp Ext_INT0     ;  
	    
Initial:	
	
	ldi	tmp1,(RAMEND)	   ;  
	out	SPL,tmp1
	
   	ldi	tmp1,0b11111111  
	out	PORTB,tmp1        
	ldi	tmp1,0b11111111		
	out	DDRB,tmp1	      
    ldi	tmp1,0b01011111
    out	PORTD,tmp1        ; PD1,PD2-
	ldi tmp1,0b01111001	
	out	DDRD,tmp1
	ldi tmp1,5          ;     1024  T0
	out	TCCR0B,tmp1	    ;    4000000/1024 = 3906,25 Hz 
	ldi tmp1,1<<INT0
	out GIMSK,tmp1      ;    
	sei
ldi tmp1,126            ;   5,0 
sts D4,tmp1
ldi tmp1,4
sts D5,tmp1
ldi tmp1,27             ;  
sts D0,tmp1
ldi tmp1,27
sts D1,tmp1
ldi tmp1,27
sts D2,tmp1
ldi tmp1,27
sts D3,tmp1
 
	rjmp Main 


Ext_INT0: ;  

sbi DDRD,sck      ;   
nop
sbi DDRD,mosi
nop
lds spi_byte,D0   ;  6   
rcall Trans_SPI
lds spi_byte,D1
rcall Trans_SPI
lds spi_byte,D2
rcall Trans_SPI
lds spi_byte,D3
rcall Trans_SPI
lds spi_byte,D4
rcall Trans_SPI
lds spi_byte,D5
rcall Trans_SPI
nop
cbi DDRD,mosi    ;   
nop
cbi DDRD,sck
inc CntSel          
cpi CntSel,1    ;      
breq EI1
cpi CntSel,2
breq EI2
cpi CntSel,3
breq EI3
reti
EI1:
rcall Text1
reti
EI2:
rcall Text2
reti
EI3:
rcall Text3
clr CntSel
reti


Trans_SPI:	;  8   SPI

ldi   Cnt,8		;     8
spi_loop:
rol   spi_byte		;   . 
brcc  low_mosi	; ,    = . 0
sbi   PortD,mosi	; MOSI = . 1
rjmp  sck_high	;      SCK
low_mosi:
cbi   PortD,mosi	; MOSI = . 0
nop	; ,     
nop
sck_high:
sbi   PortD,sck	;   SCK 
nop	; 
nop
sck_low:
cbi   PortD,sck	;   SCK
nop	; 
nop
dec   Cnt	;    1
brne  spi_loop	;  ,  Cnt   0
sbi   PortD,mosi	; MOSI     (.1)
nop
sbi   PortD,sck
ret

Main:      ;  

sbi PORTB,7  ;   
rjmp Main

Text1: ; 

cbi PORTB,7			 ;  
rcall Wait125
ldi tmp1,25
sts D0,tmp1
ldi tmp1,24
sts D1,tmp1
ldi tmp1,32
sts D2,tmp1
ldi tmp1,26
sts D3,tmp1
ret

Text3: ; 

cbi PORTB,7 		 ;  
rcall Wait125
ldi tmp1,10
sts D0,tmp1
ldi tmp1,12
sts D1,tmp1
ldi tmp1,31
sts D2,tmp1
ldi tmp1,29
sts D3,tmp1
ret

Text2: ; 

cbi PORTB,7			;  
rcall Wait125
ldi tmp1,2
sts D0,tmp1
ldi tmp1,0
sts D1,tmp1
ldi tmp1,1
sts D2,tmp1
ldi tmp1,4
sts D3,tmp1
ret


Wait125:	;   125 
ldi   tmp1,0	;   
out   TCNT0,tmp1	;  T/C0
ldi   tmp1,1<<TOV0	;     TOV0
out   TIFR,tmp1	;   TOV0
ldi   time,2	; 2   
Wait:
in    tmp1,TIFR	;    TIFR
sbrs  tmp1,TOV0	;   ,    
rjmp  Wait	;   
ldi   tmp1,1<<TOV0	;     TOV0
out   TIFR,tmp1	;   TOV0
dec   time
brne  Wait	; 2  = 125 
ret

.dseg
  
D0: .byte 1   ;    1  , D0
D1: .byte 1   ;    2  ,D1
D2: .byte 1   ;    3  , D2
D3: .byte 1   ;    4  , D3
D4: .byte 1   ;    5  , D4
D5: .byte 1   ;    6  , D5

.Exit
;*******************************************************************************
SIMBOL:    ;    (HP,G,F,E,D,C,B,A)

S0_1: .db 0b11000000, 0b11111001   ; 0,1                              
S2_3: .db 0b10100100, 0b10110000   ; 2,3                              
S4_5: .db 0b10011001, 0b10010010   ; 4,5                              
S6_7: .db 0b10000010, 0b11111000   ; 6,7                              
S8_9: .db 0b10000000, 0b10010000   ; 8,9                              
SA_b: .db 0b10001000, 0b10000011   ; A,b -10,11                             
SC_d: .db 0b11000110, 0b10100001   ; C,d -12,13
SE_F: .db 0b10000110, 0b10001110   ; E,F -14,15
SU_P: .db 0b11000001, 0b10001100   ; U,P -16,17
SL_H: .db 0b11000111, 0b10001001   ; L,H -18,19
Sr_n: .db 0b10101111, 0b10101011   ; r,n -20,21
So_t: .db 0b10100011, 0b10000111   ; o,t -22,23
Sm_g: .db 0b10111111, 0b10011100   ; "-", "  " -24,25
Sp_b: .db 0b01111111, 0b11111111   ; ".", "Blank" -   -26,27
Sc_Pi: .db 0b10100111, 0b11001000 ; "c","" -28,29
Su_Y: .db 0b11100011, 0b10010001 ; "u","Y" -30,31
S__i: .db 0b11110111, 0b11011111 ; "_","i" -32,33

Wait4000:	;   4 
ldi   tmp1,0	;   
out   TCNT0,tmp1	;  T/C0
ldi   tmp1,1<<TOV0	;     TOV0
out   TIFR,tmp1	;   TOV0
ldi   time,64	; 64   
Wait1:
in    tmp1,TIFR	;    TIFR
sbrs  tmp1,TOV0	;   ,    
rjmp  Wait1	;   
ldi   tmp1,1<<TOV0	;     TOV0
out   TIFR,tmp1	;   TOV0
dec   time
brne  Wait1	; 64  = 4 
ret

Trans_SPI:		;  8   SPI

cbi PORTD,sck
nop
sbi PORTD,sck

ldi   Cnt,8		;     8
spi_loop:
rol   spi_byte		;   . 
brcc  low_mosi	; ,    = . 0
sbi   PortD,mosi	; MOSI = . 1
rjmp  sck_high	;      SCK
low_mosi:
cbi   PortD,mosi	; MOSI = . 0
nop	; ,     
sck_high:
sbi   PortD,sck	;   SCK 
nop	; 
sck_low:
cbi   PortD,sck	;   SCK
nop	; 
dec   Cnt	;    1
brne  spi_loop	;  ,  Cnt   0
sbi   PortD,mosi	; MOSI     (.1)
nop
sbi   PortD,sck
ret

Trans_SPI2:		;  8   SPI

ldi   Cnt,8		;     8
spi_loop2:
rol   spi_byte		;   . 
brcc  low_mosi2	; ,    = . 0
sbi   PortD,mosi	; MOSI = . 1
rjmp  sck_high2	;      SCK
low_mosi2:
cbi   PortD,mosi	; MOSI = . 0
nop	; ,     
sck_high2:
sbi   PortD,sck	;   SCK 
nop	; 
sck_low2:
cbi   PortD,sck	;   SCK
nop	; 
dec   Cnt	;    1
brne  spi_loop2	;  ,  Cnt   0
sbi   PortD,mosi	; MOSI     (.1)
nop
sbi   PortD,sck
ret

Main:

sbi DDRD,mosi
nop
ldi spi_byte,10
rcall Trans_SPI
ldi spi_byte,12
rcall Trans_SPI2
ldi spi_byte,16
rcall Trans_SPI2
ldi spi_byte,17
rcall Trans_SPI2
nop
cbi DDRD,mosi
nop
rcall Wait4000
nop
sbi DDRD,mosi
nop
ldi spi_byte,1
rcall Trans_SPI
ldi spi_byte,2
rcall Trans_SPI2
ldi spi_byte,3
rcall Trans_SPI2
ldi spi_byte,4
rcall Trans_SPI2
nop
cbi DDRD,mosi
nop
rcall Wait4000

rjmp Main









